@media screen {
  div.token-overview {
    display: flex;
    flex-wrap: wrap;
  }
  div.token-overview div.panel.token {
    flex: 1;
    min-width: fit-content;
    border-right: 0;
    box-shadow: 1px 0px 0px #e5e5e5;
  }
  div.token-overview div.panel.token li.flex-group {
    display: flex;
    justify-content: space-between;
    flex-wrap: wrap;
  }
  div.token-overview div.panel.token .token-active {
    pointer-events: none;
  }
  div.token-overview div.QR {
    display: flex;
    flex-direction: column;
    align-items: center;
    padding: 2px;
  }
  div.token-overview div.QR .stream_image {
    margin: auto;
  }
  .token-title {
    border-radius: 4px 4px 0 0;
  }
  .tokens-overview-container #tokens-overview {
    display: grid;
  }
  .tokens-overview-container #tokens-overview-title {
    display: flex;
    align-items: baseline;
    justify-content: space-between;
    flex-wrap: wrap;
    gap: 10px;
  }
  .tokens-overview-container #tokens-overview-title .token-size .token-button {
    color: #000;
    margin: 0 10px;
  }
  .tokens-overview-container #tokens-overview-title button.datepicker-trigger {
    transform: translateX(-100%);
  }
  .tokens-overview-container #tokens-overview-title input.add-tokens-amount {
    width: 70px;
  }
  .tokens-overview-container #tokens-overview-title input.token-button {
    width: 70px;
  }
  .tokens-overview-container #tokens-overview-title > div > * {
    display: inline-block;
  }
  .tokens-container .scan-token {
    width: 100%;
    height: 200px;
    font-size: 500%;
  }
  .tokens-container .scan-token.inactive {
    background-color: #dd0000;
  }
  .tokens-container .scan-token.active {
    background-color: #00dd00;
  }
}
@media print {
  #tokens-overview-title {
    display: none;
  }
}
